package kr.co.vividnext.sodalive.live.recommend

import com.querydsl.core.types.Projections
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.recommend.QRecommendLiveCreatorBanner.recommendLiveCreatorBanner
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
class LiveRecommendRepository(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    fun getRecommendLive(memberId: Long, isAdult: Boolean): List<GetRecommendLiveResponse> {
        val dateNow = LocalDateTime.now()

        var where = recommendLiveCreatorBanner.startDate.loe(dateNow)
            .and(recommendLiveCreatorBanner.endDate.goe(dateNow))

        if (!isAdult) {
            where = where.and(recommendLiveCreatorBanner.isAdult.isFalse)
        }

        return queryFactory
            .select(
                Projections.constructor(
                    GetRecommendLiveResponse::class.java,
                    recommendLiveCreatorBanner.image.prepend("/").prepend(cloudFrontHost),
                    recommendLiveCreatorBanner.creator.id
                )
            )
            .from(recommendLiveCreatorBanner)
            .where(where)
            .orderBy(recommendLiveCreatorBanner.orders.asc())
            .fetch()
    }

    fun getOnAirRecommendChannelList(
        memberId: Long,
        isAdult: Boolean
    ): List<GetRecommendChannelResponse> {
        var where = member.role.eq(MemberRole.CREATOR)
            .and(member.isActive.isTrue)

        if (!isAdult) {
            where = where.and(liveRoom.isAdult.isFalse)
        }

        return queryFactory
            .select(
                Projections.constructor(
                    GetRecommendChannelResponse::class.java,
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(cloudFrontHost),
                    Expressions.asBoolean(true)
                )
            )
            .from(liveRoom)
            .rightJoin(liveRoom.member, member)
            .where(
                where
                    .and(liveRoom.isActive.isTrue)
                    .and(liveRoom.channelName.isNotNull)
                    .and(liveRoom.channelName.isNotEmpty)
            )
            .groupBy(member.id)
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .limit(20)
            .fetch()
    }

    fun getRecommendChannelList(
        memberId: Long,
        withOutCreatorList: List<Long>,
        limit: Long
    ): List<GetRecommendChannelResponse> {
        val where = member.role.eq(MemberRole.CREATOR)
            .and(member.isActive.isTrue)

        return queryFactory
            .select(
                Projections.constructor(
                    GetRecommendChannelResponse::class.java,
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(cloudFrontHost),
                    Expressions.asBoolean(false)
                )
            )
            .from(member)
            .where(where.and(member.id.notIn(withOutCreatorList)))
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .limit(limit)
            .fetch()
    }
}
