package kr.co.vividnext.sodalive.member

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface MemberRepository : JpaRepository<Member, Long>, MemberQueryRepository {
    fun findByEmail(email: String): Member?
    fun findByNickname(nickname: String): Member?
}

interface MemberQueryRepository {
    fun findByPushToken(pushToken: String): List<Member>
    fun findByNicknameAndOtherCondition(nickname: String, memberId: Long): List<Member>
    fun findCreatorByIdOrNull(memberId: Long): Member?
}

@Repository
class MemberQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : MemberQueryRepository {
    override fun findByPushToken(pushToken: String): List<Member> {
        return queryFactory
            .selectFrom(member)
            .where(member.pushToken.eq(pushToken))
            .fetch()
    }

    override fun findByNicknameAndOtherCondition(nickname: String, memberId: Long): List<Member> {
        return queryFactory
            .selectFrom(member)
            .where(
                member.nickname.containsIgnoreCase(nickname)
                    .and(member.id.ne(memberId))
                    .and(member.role.ne(MemberRole.ADMIN))
                    .and(member.role.ne(MemberRole.AGENT))
            )
            .fetch()
    }

    override fun findCreatorByIdOrNull(memberId: Long): Member? {
        return queryFactory
            .selectFrom(member)
            .where(
                member.id.eq(memberId)
                    .and(member.role.eq(MemberRole.CREATOR))
            )
            .fetchFirst()
    }
}
